;************************************************************************************************
;
;	AVRUSB162i - Interactive AT90USB162 (AVR) Full Speed USB implementation
;
;	A general purpose HID Device based on the AT90USB82/162 AVR microcontrollers.  When attached
;	to a USB port, this device will automatically load the Windows HID-Compliant device driver.
;
;	The Descriptor is designed to match up with the GenericHid application available from
;	www.lvr.com.  GenericHid sends two bytes via Endpoint2 and reads two bytes via Endpoint1.  The
;	firmware echos back the two bytes received.  Because Endpoint1 is continually polled by the
;	host, the F_DATA_READY flag is used to signal when new bytes have been received by the device.
;	If no new data has been received, the device responds to an Endpoint1 request with a Zero
;	Length Packet.
;
;	Endpoints are defined as follows:
;
;		Endpoint0 (Control)	- 64 bytes
;		Endpoint1 (IN - Interrupt) - 32 bytes
;		Endpoint2 (OUT - Interrupt) - 32 bytes
;
;	Serial Port Macro Guidelines
;
;	You can place the serial port macros just about anywhere but recognize that some USB register
;	status bits may change state during the time it takes to transmit a character.  In addition,
;	there are some timing critical areas in the USB protocol.  As a general rule, you should not
;	place a serial port macro before the following macros to avoid timing related problems: 
;
;		ClearSETUP
;		SetSTALL
;		ClearIN
;		ClearOUT  
;
;	The Std_Set_Address routine is timing critical until the completion of the EnableAddress macro.
;	Use a mov instruction to put the register value aside so that you can send it to the serial port
;	after the timing critical logic has finished.
;
;	EEPROM is not used.
;
;	16MHz external crystal, startup: 16K CK, 4.1 ms
;
;	Author:	Richard Dzioba	
;
;   Version: December 21, 2009
;
;************************************************************************************************

.include "usb162def.inc"
.include "STANDARD_Macros.inc"
.include "PORT_Macros.inc"
.include "USB_Macros.inc"
.include "USART_Macros.inc"

;---------------------------------------------------------------------
;	Equates
;---------------------------------------------------------------------
.EQU	EP0_PACKET_SIZE			= 0x40		; Endpoint0 packet size (64 bytes)

; Flag register with bit definitions:
.EQU	FLAGS					= GPIOR0	; General Purpose flag register
.EQU	F_CONFIGURED			= 0			; set when the Host configures the device
.EQU	F_SUSPENDED				= 1			; set to enter sleep mode after detecting Suspend interrupt
.EQU	F_DATA_READY			= 2			; set to indicate that new data is available for an IN transfer
.EQU	F_FLAG3					= 3			; unassigned
.EQU	F_FLAG4					= 4			; unassigned
.EQU	F_FLAG5					= 5			; unassigned
.EQU	F_FLAG6					= 6			; unassigned
.EQU	F_FLAG7					= 7			; unassigned

; Control Setup Get_Descriptor codes:	
.EQU 	GD_DEVICE				= 0x01
.EQU 	GD_CONFIGURATION		= 0x02
.EQU 	GD_STRING				= 0x03
.EQU 	GD_HID	            	= 0x21
.EQU 	GD_REPORT	        	= 0x22

;---------------------------------------------------------------------
;	Register Definitions
;---------------------------------------------------------------------
; R0 - is reserved for the LPM instruction

; 8 byte Setup Packet registers:
.DEF	bmRequestType			= R1
.DEF	bRequest				= R2
.DEF	wValueL					= R3
.DEF	wValueH					= R4
.DEF	wIndexL					= R5
.DEF	wIndexH					= R6
.DEF	wLengthL				= R7
.DEF	wLengthH				= R8

.DEF	sregTemp				= R9		; temporary store for SREG during interrupt processing
.DEF	descLenL				= R10		; descriptor length - low byte
.DEF	descLenH				= R11		; descriptor length - high byte

.DEF	temp					= R16		; general purpose, also used internally by most macros
.DEF	usartByte				= R17		; USART data byte
.DEF	dataByte1				= R18		; data transfer byte1
.DEF	dataByte2				= R19		; data transfer byte2
.DEF	packetIndex				= R20		; used to keep track of the total number of descriptor bytes sent per packet

; R30, R31 - is reserved for the Z register

;*********************************************************************
;	Data Segment - SRAM Storage (512 bytes)
;*********************************************************************
.DSEG
.ORG	SRAM_START

;*********************************************************************
;	Reset and Interrupt Vectors
;*********************************************************************
.CSEG
.ORG	0x00
	jmp		Reset
	jmp		Ret_I;	EXT_INT0 			; External Interrupt Request 0
	jmp		Ret_I;	EXT_INT1 			; External Interrupt Request 1
	jmp		Ret_I;	EXT_INT2 			; External Interrupt Request 2
	jmp		Ret_I;	EXT_INT3 			; External Interrupt Request 3
	jmp		Ret_I;	EXT_INT4 			; External Interrupt Request 4
	jmp		Ret_I;	EXT_INT5 			; External Interrupt Request 5
	jmp		Ret_I;	EXT_INT6 			; External Interrupt Request 6
	jmp		Ret_I;	EXT_INT7			; External Interrupt Request 7
	jmp		Ret_I;	PC_INT0				; Pin Change Interrupt Request 0
	jmp		Ret_I;	PC_INT1 			; Pin Change Interrupt Request 1
	jmp		USB_Device	 				; USB Device Interrupt request
	jmp		USB_Endpoint				; USB Endpoint Interrupt request
	jmp		Ret_I;	WDT 				; Watchdog Time-out Interrupt
	jmp		Ret_I;	TIMER1_CAPT 		; Timer/Counter1 Capture Event
	jmp		Ret_I;	TIMER1_COMPA 		; Timer/Counter1 Compare Match A
	jmp		Ret_I;	TIMER1_COMPB 		; Timer/Counter1 Compare Match B
	jmp		Ret_I;	TIMER1_COMPC 		; Timer/Counter1 Compare Match C
	jmp		Ret_I;	TIMER1_OVF			; Timer/Counter1 Overflow
	jmp		Ret_I;	TIMER0_COMPA 		; Timer/Counter0 Compare Match A
	jmp		Ret_I;	TIMER0_COMPB 		; Timer/Counter0 Compare match B
	jmp		Ret_I;	TIMER0_OVF			; Timer/Counter0 Overflow
	jmp		Ret_I;	SPI_COMP			; SPI, STC SPI Serial Transfer Complete
	jmp		Ret_I;	USART1_RX 			; USART1 Rx Complete
	jmp		Ret_I;	USART1_UDRE			; USART1 Data Register Empty
	jmp		Ret_I;	USART1TX_USART1		; Tx Complete
	jmp		Ret_I;	ANALOG_COMP 		; Analog Comparator
	jmp		Ret_I;	EE_READY 			; EEPROM Ready
	jmp		Ret_I;	SPM_READY			; Store Program Memory Ready
Ret_I:	
	reti

;*********************************************************************
;
;	Reset - Initialization
;
;*********************************************************************
Reset:
;	Initialize stack pointer:
	ldi		temp,HIGH(RAMEND)
	out		SPH,temp	
	ldi		temp,LOW(RAMEND)
	out		SPL,temp

;	Disable clock prescaler:
	ldi		temp,(1<<CLKPCE)			; enable main clock prescale update
	sts		CLKPR,temp
	ldi		temp,(0<<CLKPCE)			; update main clock prescaler to /1 to override fuse setting
	sts		CLKPR,temp		

; 	Configure PORTB						; PI7|PI6|PI5|PI4|PI3 |PI2 |PI1 |PI0
	ldi		temp,0b11111111				; PB7|PB6|PB5|PB4|PB3 |PB2 |PB1 |PB0
	out		DDRB,temp					;  O | O | O | O |MISO|MOSI|SCLK| O
	ldi		temp,0b00000000				; set pullups
	out		PORTB,temp

; 	Configure PORTC						; IN4|IN8|IN9|P10| - |P11| - | -
	ldi		temp,0b11110100				; PC7|PC6|PC5|PC4|PC3|PC2|PC1|PC0 <- PORTC.0 must be zero for the external oscillator to run!!
	out		DDRC,temp					;  O | O |LED| O | - | O |RST|XT2
	ldi		temp,0b00000000				; set pullups
	out		PORTC,temp

; 	Configure PORTD						; IN7|IN6|P12|IN5|IN3|IN2|IN1|IN0
	ldi		temp,0b01111111				; PD7|PD6|PD5|PD4|PD3|PD2|PD1|PD0
	out		DDRD,temp					; HWB| O | O | O |TXD| O | O | O
	ldi		temp,0b10000000				; set pullups
	out		PORTD,temp

;	Configure CPU sleep mode:
	ConfigureSLEEPMode

;	Disable comparator to save power during Suspend
	DisableCOMPARATOR

; 	Initialize USART:
	ConfigureUSART_2x 115200			; baud rate is CPU clock speed dependent - refer to the datasheet

;	Indicate that a CPU reset has been performed
	PutChrI	'$',TRUE

; 	Initialize USB module:
	EnableRegulator
	EnableUSB
	SetPLLPrescale
	EnablePLL
	EnableUSBClock
	AttachUSB	
											
	EnableRESET

	sei									; enable global interrupts

;*********************************************************************
;
;	Main Loop
;
;*********************************************************************
Main_Loop:
	sbic	FLAGS, F_SUSPENDED			; check if Suspend interrupt received
	rcall	Power_Down					; if yes, initiate power down
	sbis	FLAGS, F_CONFIGURED			; wait until device is configured
	rjmp	Main_Loop

	rjmp	Main_Loop

;---------------------------------------------------------------------
;	Enter sleep mode and wait for USB Wakeup interrupt
;---------------------------------------------------------------------
Power_Down:
	LED_OFF								; turn off "Configured" LED
	EnableSLEEP
	sleep								; power down and wait for interrupt
	DisableSLEEP
	ClrFLAG	F_SUSPENDED
	ret

;*********************************************************************
;
;	USB Device Interrupt Handler (UDINT)
;
;*********************************************************************
USB_Device:
	in		sregTemp,SREG
	push	temp

	rcall	UD_Process_Interrupt

	pop		temp
	out		SREG,sregTemp
	reti

UD_Process_Interrupt:
	lds		temp,UDINT
	sbrc	temp,SUSPI
	rjmp	UD_Suspend
	sbrc	temp,WAKEUPI
	rjmp	UD_Wakeup
	sbrc	temp,EORSTI
	rjmp	UD_Reset
	ret									; should never reach here

;---------------------------------------------------------------------
;	USB Device interrupt subroutines
;---------------------------------------------------------------------
UD_Suspend:
	ClearSUSPEND
	DisableSUSPEND
	EnableWAKEUP
	DisableUSBClock
	DisablePLL
	SetFLAG	F_SUSPENDED					; signal the main loop to enter power down mode
	ret

UD_Wakeup:
	EnablePLL
	EnableUSBClock
	ClearWAKEUP
	DisableWAKEUP
	EnableSUSPEND
	ret

UD_Reset:
	ClearRESET
	ClearSUSPEND
	DisableSUSPEND
	EnableWAKEUP
	rcall	Configure_Endpoints
	ret

;---------------------------------------------------------------------
;	Configure Endpoints
;---------------------------------------------------------------------
Configure_Endpoints:
;	Endpoint 0 (Control Endpoint):
	ResetEndpoint	(EP_0)
	SelectEndpoint	(EP_0)
	EnableEndpoint
	SetEndpointType	(EP_CONTROL | EP_OUT)
	SetEndpointSize	(EP_SIZE64 | EP_ONE_BANK)
	AllocateEndpointMemory
	EnableSETUP

;	Endpoint 1 (IN Endpoint):
	ResetEndpoint	(EP_1)
	SelectEndpoint	(EP_1)
	EnableEndpoint
	SetEndpointType	(EP_INTERRUPT | EP_IN)
	SetEndpointSize	(EP_SIZE32 | EP_ONE_BANK)
	AllocateEndpointMemory
	EnableIN

;	Endpoint 2 (OUT Endpoint):
	ResetEndpoint	(EP_2)
	SelectEndpoint	(EP_2)
	EnableEndpoint
	SetEndpointType	(EP_INTERRUPT | EP_OUT)
	SetEndpointSize	(EP_SIZE32 | EP_ONE_BANK)
	AllocateEndpointMemory
	EnableOUT
;
	ret

;*********************************************************************
;
;	USB Endpoint Interrupt Handler (UEINT)
;
;*********************************************************************
USB_Endpoint:
	in		sregTemp,SREG
	push	temp

	rcall	UE_Process_Interrupt

	pop		temp
	out		SREG,sregTemp
	reti

UE_Process_Interrupt:
	lds		temp,UEINT
	sbrc	temp,EPINT0
	rjmp	UE_Endpoint0
	sbrc	temp,EPINT1
	rjmp	UE_Endpoint1
	sbrc	temp,EPINT2
	rjmp	UE_Endpoint2
	ret									; should never reach here

;*********************************************************************
;
;	ENDPOINT 0 - Control Transfer (Host -> Device)
;
;*********************************************************************
UE_Endpoint0:
	SelectEndpoint EP_0

; 	Read the 8 byte Setup packet:
	ReadByte	bmRequestType
	ReadByte	bRequest
	ReadByte	wValueL
	ReadByte	wValueH
	ReadByte	wIndexL
	ReadByte	wIndexH
	ReadByte	wLengthL
	ReadByte	wLengthH

;	Check if bRequest is out of bounds:
	mov		temp,bRequest				; get the request type from the Setup Packet
	cpi		temp,0x0C					; it must not be 0x0C or higher
	brsh	Stall						; if so, report error

;	Determine if Setup or HID request (USB Section 9.3 Table 9-2):
	sbrs	bmRequestType,5				; look at bit b5 only
	rjmp	UE_Std_Setup_Request		; b5 = 0 -> Setup
	rjmp	UE_Hid_Class_Request		; b5 = 1 -> HID

;---------------------------------------------------------------------
;	Process Standard Setup Requests
;---------------------------------------------------------------------
UE_Std_Setup_Request:
	Ldiw	Z,UE_Std_Jump_Table
	mov		temp,bRequest				; get the Standard request type from the Setup Packet
	Addw	Z,temp						; get jump table offset
	ijmp								; go to the subroutine
UE_Std_Jump_Table:
	rjmp	Stall						; 00 - Get_Status (not implemented)		
	rjmp	Stall						; 01 - Clear_Feature (not implemented)
	rjmp	Stall						; 02 - Reserved
	rjmp	Stall						; 03 - Set_Feature (not implemented)
	rjmp	Stall						; 04 - Reserved
	rjmp	Std_Set_Address				; 05 - Set_Address
	rjmp	Std_Get_Descriptor			; 06 - Get_Descriptor
	rjmp	Stall						; 07 - Set_Descriptor (not implemented)
	rjmp	Stall						; 08 - Get_Configuration (not implemented)
	rjmp	Std_Set_Configuration		; 09 - Set_Configuration
	rjmp	Stall						; 0A - Get_Interface (not implemented)
	rjmp	Stall						; 0B - Set_Interface (not implemented)

;---------------------------------------------------------------------
;	Process HID Class Requests
;---------------------------------------------------------------------
UE_HID_Class_Request:
	Ldiw	Z,UE_HID_Jump_Table
	mov		temp,bRequest				; get the HID request type from the Control Packet
	Addw	Z,temp						; get jump table offset
	ijmp								; go to the subroutine
UE_HID_Jump_Table:
	rjmp	Stall						; 00 - Reserved			
	rjmp	Hid_Get_Report				; 01 - Get_Report
	rjmp	Stall						; 02 - Get_Idle (not implemented)
	rjmp	Stall						; 03 - Get_Protocol (not implemented)
	rjmp	Stall						; 04 - Reserved
	rjmp	Stall						; 05 - Reserved
	rjmp	Stall						; 06 - Reserved
	rjmp	Stall						; 07 - Reserved
	rjmp	Stall						; 08 - Reserved
	rjmp	Hid_Set_Report				; 09 - Set_Report
	rjmp	Stall						; 0A - Set_Idle (not implemented)
	rjmp	Stall						; 0B - Set_Protocol (not implemented)

;---------------------------------------------------------------------
; 	Stall - Unsupported Request
;---------------------------------------------------------------------
Stall:
	SetSTALL							; set Stall BEFORE acknowledging Setup!
	ClearSETUP							; ACK Setup
	ClearSTALL							; reset the Stall flag

	rcall	Display_Setup_Stall			; display setup packet with "Stall" indicator
	ret

;---------------------------------------------------------------------
; 	Get Descriptor (0x06)
;---------------------------------------------------------------------
Std_Get_Descriptor:
	mov		temp,wValueH				; get Descriptor Type from the Setup Packet

	cpi		temp,GD_DEVICE				; 0x01
	breq	Std_Device_Descriptor
	cpi		temp,GD_CONFIGURATION		; 0x02
	breq	Std_Config_Descriptor
	cpi		temp,GD_STRING				; 0x03
	breq	Std_String_Descriptor
	cpi		temp,GD_HID					; 0x21
	breq	Std_Class_Descriptor	
	cpi		temp,GD_REPORT				; 0x22
	breq	Std_Report_Descriptor
	rjmp	Stall						; else, unknown descriptor request

Std_Device_Descriptor:
	Ldiw	Z,(DD*2)
	lpm		descLenL,Z
	rjmp	Send_Descriptor

Std_Config_Descriptor:
	Ldiw	Z,(CD*2)+2
	lpm		descLenL,Z
	Ldiw	Z,(CD*2)
	rjmp	Send_Descriptor

Std_String_Descriptor:
	mov		temp,wValueL				; get the String Index from the Setup packet
	cpi		temp,0
	breq	Std_String_Language
	cpi		temp,1
	breq	Std_String_Manufacturer
	cpi		temp,2
	breq	Std_String_Product
	cpi		temp,3
	breq	Std_String_SerialNumber
	rjmp	Stall

Std_String_Language:
	Ldiw	Z,(SD0*2)
	lpm		descLenL,Z
	rjmp	Send_Descriptor

Std_String_Manufacturer:
	Ldiw	Z,(SD1*2)
	lpm		descLenL,Z
	rjmp	Send_Descriptor

Std_String_Product:
	Ldiw	Z,(SD2*2)
	lpm		descLenL,Z
	rjmp	Send_Descriptor

Std_String_SerialNumber:
	Ldiw	Z,(SD3*2)
	lpm		descLenL,Z
	rjmp	Send_Descriptor

Std_Class_Descriptor:
	Ldiw	Z,(HD*2)
	lpm		descLenL,Z
	rjmp	Send_Descriptor

Std_Report_Descriptor:
	Ldiw	Z,(HD*2)+7
	lpm		descLenL,Z
	Ldiw	Z,(RD*2)
	rjmp	Send_Descriptor

;---------------------------------------------------------------------
; 	Send Descriptor to Host (in multiple packets as required).
;---------------------------------------------------------------------
Send_Descriptor:
	ClearSETUP							; ACK Setup packet

;	Send lesser of actual length and the host requested length (see USB Section 9.4.3).
	clr		descLenH					; our descriptor LengthH is always zero
	Minw	descLenH, descLenL, wLengthH, wLengthL

	rcall	Display_Setup_Packet		; display setup packet

Sd_Begin_Packet:
	ldi		packetIndex,EP0_PACKET_SIZE

Sd_Write_Bytes:
	lpm		temp,Z+	

	WriteByte	temp					; write descriptor byte to EP FIFO

	dec		descLenL
	breq	Sd_Send_Packet				; reached end of descriptor?
	dec		packetIndex
	brne	Sd_Write_Bytes				; reached end of FIFO?

Sd_Send_Packet:
	ClearIN								; send packet
	WaitINReady							; wait until the Status stage is done 

	tst		descLenL					; has the entire descriptor been sent?
	brne	Sd_Begin_Packet				; no - begin another packet

	WaitOUTReceived
	ClearOUT
	ret

;---------------------------------------------------------------------
; 	Set Address (0x05)
;---------------------------------------------------------------------
Std_Set_Address:
	SetAddress	wValueL					; low byte contains the address assigned
	ClearSETUP							; must respond within 50ms from receipt of RXSTPI
	ClearIN								; send Zero Length Packet for Status stage
	WaitINReady							; wait until the Status stage is done
	EnableAddress						; must be completed within 2ms from clearing TXINI

	rcall	Display_Setup_Packet		; display setup packet
	ret
	
;---------------------------------------------------------------------
; 	Set Configuration (0x09)
;---------------------------------------------------------------------
Std_Set_Configuration:
	ClearSETUP							; ACK Setup
	ClearIN								; only one configuration, so just acknowledge request with ZLP.
	WaitINReady							; wait until the Status phase is done

	SetFLAG	F_CONFIGURED				; device is now "Configured"
	LED_ON								; turn on Configured LED

	rcall	Display_Setup_Packet		; display setup packet
	ret

;---------------------------------------------------------------------
; 	HID Get Report (0x01) - Control Read (Device -> Host)
;	wValueH = 0x01 - Get Report (In)
;	wValueH = 0x03 - Get Report (Feature)
;---------------------------------------------------------------------
Hid_Get_Report:
	ClearSETUP							; ACK Setup
	WaitINReady

	WriteByte	dataByte1
	WriteByte	dataByte2

	rcall	Display_Report_Data			; display the request packet and data bytes

	ClearIN
	WaitOUTReceived
	ClearOUT
	ret

;---------------------------------------------------------------------
; 	HID Set Report (0x09) - Control Write (Host -> Device)
;   wValueH = 0x02 - Set Report (Out)
;	wValueH = 0x03 - Set Report (Feature)
;---------------------------------------------------------------------
Hid_Set_Report:
	ClearSETUP							; ACK Setup
	WaitOUTReceived

	ReadByte	dataByte1
	ReadByte	dataByte2

	rcall	Display_Report_Data			; display the request packet and data bytes

	ClearOUT
	ClearIN
	WaitINReady
	ret

;*********************************************************************
;
;	ENDPOINT 1 - IN Transfer (Device -> Host)
;
;*********************************************************************
UE_Endpoint1:
	SelectEndpoint EP_1
	ClearIN
	WaitReadWriteAllowed				; wait until endpoint data is ready

	EP1_POLL							; toggle pin to check Endpoint polling frequency with oscilloscope

	sbis		FLAGS,F_DATA_READY		; send data only if it has changed
	rjmp		UE_E1_Exit				; otherwise reply with a ZLP

	WriteByte	dataByte1				; write bytes to EP1 FIFO
	WriteByte	dataByte2

	ClrFLAG		F_DATA_READY			; reset data available flag

;	Display bytes to be sent to the host:
	PutChrI		'E',FALSE
	PutChrI		'1',FALSE
	PutChrI		':',FALSE
	PutHex		dataByte1,FALSE
	PutHex		dataByte2,TRUE

UE_E1_Exit:
	ClearFIFO							; send endpoint to host
	ret

;*********************************************************************
;
;	ENDPOINT 2 - OUT Transfer (Host -> Device)
;
;*********************************************************************
UE_Endpoint2:
	SelectEndpoint EP_2
	ClearOUT
	WaitReadWriteAllowed				; wait until endpoint data is ready

	ReadByte	dataByte1				; read bytes from EP2 FIFO
	ReadByte	dataByte2

	SetFlag		F_DATA_READY			; set flag to indicate data received from host

;	Display bytes received from the host:
	PutChrI		'E',FALSE
	PutChrI		'2',FALSE
	PutChrI		':',FALSE
	PutHex		dataByte1,FALSE
	PutHex		dataByte2,TRUE

UE_E2_Exit:
	ClearFIFO							; clear the endpoint
	ret

;*********************************************************************
;	Display Routines - Send data to host via serial port
;*********************************************************************
; 	Display the Standard Request packet.
Display_Setup_Packet:
	rcall	Display_Packet_Data
	PutChrI	NULL,TRUE					; put a new line (i.e., "\n").	
	ret

; 	Display the Standard Request packet including the "Stall" indicator.
Display_Setup_Stall:
	rcall	Display_Packet_Data
	PutChrI	' ',FALSE
	PutChrI	'S',TRUE
	ret

;	Display the HID Request packet including the Set/Get data bytes.
Display_Report_Data:
	rcall	Display_Packet_Data
	PutChrI	':',FALSE
	PutHex	dataByte1,FALSE
	PutHex	dataByte2,TRUE	
	ret

;	Display the Request packet, keep on same line.
Display_Packet_Data:
	PutHex	bmRequestType,FALSE
	PutChrI	' ',FALSE
	PutHex	bRequest,FALSE
	PutChrI	' ',FALSE
	PutHex	wValueL,FALSE
	PutHex	wValueH,FALSE
	PutChrI	' ',FALSE
	PutHex	wIndexL,FALSE
	PutHex	wIndexH,FALSE
	PutChrI	' ',FALSE
	PutHex	wLengthL,FALSE
	PutHex	wLengthH,FALSE
	ret

;********************************************************************
;	USART Functions
;********************************************************************
.include "USART_Functions.inc"

;********************************************************************
;	Descriptor
;********************************************************************
.include "DESCRIPTOR_Generic_HID.inc"
//.include "DESCRIPTOR_Media_HID.inc"
//.include "DESCRIPTOR_Keyboard_HID.inc"
//.include "DESCRIPTOR_Serial_CDC.inc"




