;********************************************************************************************************
;
;	Generic HID Descriptor - In, Out and Feature Reports.
;
;	Allows an AT90USB162 to enumerate as a HID device, using the Vendor Defined usage tables.  Uses the
;	drivers included with Windows.
;
;	Endpoints are defined as follows:
;
;		Control Endpoint	64 bytes	(EP0_PACKET_SIZE)
;		Endpoint 1			32 bytes 	IN endpoint (device -> host)
;		Endpoint 2			32 bytes	OUT endpoint (host -> device)
;
;	References to the USB2.0 and HID1.11 specifications are provided.  The specifications are downloadable 
;	from http://www.usb.org/developers/docs.
;
; 	Note: The compiler organizes memory in words.  Descriptor values need to be written as two bytes per
;	line, otherwise the assembler will pad each byte to make it a word.
;
;	Version: November 21, 2009
;
;********************************************************************************************************

;	Device_Descriptor (USB Section 9.6.1)
DD:
	.DB 18,0x01					; bLength=18d							bDescriptorType (Device)
	.DW 0x0200					; bcdUSB (USB Rev 2.00)
	.DW 0x0000					; bDeviceClass							bDeviceSubClass 
	.DB 0x00,EP0_PACKET_SIZE	; bDeviceProtocol						bMaxPacketSizeEP0 (default=64 bytes)
	.DW 0x03EB					; idVendor (Atmel)
	.DW 0x21FF					; idProduct (Atmel Application Note AVR309)
	.DW 0x0000					; bcdDevice
	.DB 1,2,3,1					; iManufacturer, iProduct, iSerialNumber, bNumConfigurations

;	Configuration Descriptor (USB Section 9.6.3)
;	Interface Descriptor (USB Section 9.6.5)
;
;	The Interface Descriptor must directly follow the Configuration Descriptor.
CD:
	.DB 9,0x02					; bLength=9d							bDescriptorType (Configuration)
	.DB ((CD_End - CD)*2 + (HD_End - HD)*2) - 1, 0; 					wTotalLengthL=CD+ID+HD+ED1+ED2-PadByte, wTotalLengthH
	.DB 0x01,0x01				; bNumInterfaces (1)					bConfigValue
	.DB 0x00,0xC0				; iConfiguration						bmAttributes (b7=1 b6=self-powered b5=RWU not supported)
	.DB 100/2,9					; MaxPower (100mA in 2mA units)			INTERFACE Descriptor, Length=9d
	.DB 0x04,0					; bDescriptorType (Interface)			bInterfaceNumber (0)
	.DB 0x00,2					; bAlternateSetting						bNumEndpoints (2)
	.DB 0x03,0x00				; bInterfaceClass (HID)					bInterfaceSubClass
	.DB 0x00,0x00				; bInterfaceProtocol					iInterface
CD_End:

;	HID Descriptor (HID Section 6.2.1)
;	Endpoint Descriptors (USB Section 9.6.6)
;
;	The Endpoint Descriptors must directly follow the Hid Descriptor.
HD:
	.DB 9,0x21					; bLength=9d							bDescriptorType (HID)
	.DW 0x0111					; bcdHID (HID Rev 1.11)
	.DB 0x00,1					; bCountryCode (none)					bNumDescriptors (1)
	.DB 0x22,(RD_End - RD)*2	; bDescriptorType (Report)				wReportDescriptorLengthL

	.DB 0x00,7					; wReportDescriptorLengthH				ENDPOINT1 Descriptor bLength=7d
	.DB 0x05,0x81				; bDescriptorType (Endpoint)			bEndpointAddress (IN EP1=0x80|0x01)	
	.DB 0x03,0x20				; bmAttributes (Interrupt)				wMaxPacketSizeL (32 bytes)
	.DB 0x00,0x0A				; wMaxPacketSizeH						bInterval (10 msec)

	.DB 7,0x05					; ENDPOINT2 Descriptor bLength=7d		bDescriptorType (Endpoint)	
	.DB 0x02,0x03				; bEndpointAddress (OUT EP2=0x00|0x02)	bmAttributes (Interrupt)
	.DB 0x20,0x00				; wMaxPacketSizeL (32 bytes)			wMaxPacketSizeH
	.DB 0x0A,0x00				; bInterval (10 msec)					*Pad Byte*
HD_End:

;	Report Descriptor (HID Section 6.2.2)
;
;	The column on the far right lists the actual bytes for each usage.  Usage bytes are
;	stored as two bytes per memory location.  The first usage is three bytes and causes
;	the wrap-around.
RD:
	.DB 0x06,0x00				; Usage Page (Vendor Defined)			06 00 FF
	.DB 0xFF,0x09				; Usage (Vendor Usage)					09 01
	.DB 0x01,0xA1				; Collection (Application)				A1 01

;	Input Report:
	.DB 0x01,0x09				; 	Usage (Vendor)						09 01
	.DB 0x01,0x15				;	Logical Minimum (-127)				15 00
	.DB 0x00,0x25				;   Logical Maximum (128)				25 01
	.DB 0x01,0x75				;	Report Size (8)						75 08
	.DB 0x08,0x95				;	Report Count (2)					95 02
	.DB 0x02,0x81				;	Input (Data, Var, Abs)				81 02

;	Output Report:
	.DB 0x02,0x09				;	Usage (Vendor)						09 01
	.DB 0x01,0x15				;   Logical Minimum (-127)				15 00
	.DB	0x00,0x25				;	Logical Maximum (128)				25 01
	.DB	0x01,0x75				;	Report Size (8)						75 08
	.DB	0x08,0x95				; 	Report Count (2)					95 02
	.DB	0x02,0x91				;	Output (Data, Var, Abs)				91 02

; 	Feature Report:
	.DB	0x02,0x09				;	Usage (Vendor)						09 01
	.DB	0x01,0x15				;   Logical Minimum (-127)				15 00
	.DB	0x00,0x25				;	Logical Maximum (128)				25 01
	.DB	0x01,0x75				;	Report Size (8)						75 08
	.DB	0x08,0x95				; 	Report Count (2)					95 02
	.DB	0x02,0xB1				;	Feature (Data, Var, Abs)			B1 02

	.DB	0x02,0xC0				; End Collection						C0
RD_End:

; 	String Descriptors (USB Section 9.6.7).  All strings are 2 byte Unicode
;
;	Language:
SD0:
	.DB (SD0_End - SD0)*2,0x03		; bLength							bDescriptorType (String)
	.DW 0x0409						; wLangID (US English)	
SD0_End:	

;	Manufacturer ID:
SD1:
	.DB (SD1_End - SD1)*2,0x03		; bLength							bDescriptorType (String)
	.DW 'G','E','N','E','R','I','C'
SD1_End:

;	Product ID:
;	This string is displayed in the Windows "bubble" when the driver is first installed.
SD2:
	.DB (SD2_End - SD2)*2,0x03		; bLength							bDescriptorType (String)
	.DW 'A','V','R','U','S','B','1','6','2','i'
SD2_End:

;	Serial Number (Device Instance ID):
SD3:
	.DB (SD3_End - SD3)*2,0x03		; bLength							bDescriptorType (String)
	.DW '0','0','0','1'
SD3_End:
