;********************************************************************************************************
;
;	MEDIA HID Descriptor - In and Out Reports (2 bytes each).  No Feature Report implemented.
;
;	Allows an AT90USB162 to enumerate as a Media HID device, using the Consumer usage tables.  No
;	custom drivers are required for this device to operate using Windows XP.
;
;	Endpoints are defined as follows:
;
;		Control Endpoint	64 bytes	(EP0_PACKET_SIZE)
;		Endpoint 1			32 bytes 	IN endpoint (device -> host)
;		Endpoint 2			32 bytes	OUT endpoint (host -> device)
;
;	References to the USB2.0 and HID1.11 specifications are provided.  The specifications are downloadable 
;	from http://www.usb.org/developers/docs.
;
;	NOTE: The AVR compiler arranges memory in words.  If a descriptor length is an uneven number
;	of bytes, a "pad byte" must be included explicitly (if not, the compiler will add one and will generate
;	a warning message).  The descriptor length calculation must take the pad byte into consideration.
;	Length calculations are in words and are multiplied by 2 to get the number of bytes.
;
;	To use the Media descriptor, send two bytes via the IN endpoint whenever your device changes one
;	of the media functions.  The first byte must contain a "1" in the bit position that corresponds to
;	the media function desired.  The second byte is always zeros with this descriptor.  The next
;	IN transaction must send both bytes as all zeros.  For example, to toggle the Mute function On,
;	send two bytes: 0x01, 0x00.  On the next IN transaction, send two bytes: 0x00, 0x00.  To toggle Mute Off,
;	send the same sequence (i.e., 0x01, 0x00, 0x00, 0x00).
;
;	The OUT endpoint is not used by the Media descriptor.
;
;	The HID Descriptor Tool is used to define the Report Descriptor.  The tool is found at:
;	www.usb.org/developers/hidpage/dt2_4.zip
;
;	Version: November 25, 2009
;
;********************************************************************************************************

;	Device_Descriptor (USB Section 9.6.1)
DD:
	.DB 18,0x01					; bLength=18d							bDescriptorType (Device)
	.DW 0x0200					; bcdUSB (USB Rev 2.00)
	.DW 0x0000					; bDeviceClass							bDeviceSubClass 
	.DB 0x00,EP0_PACKET_SIZE	; bDeviceProtocol						bMaxPacketSizeEP0 (default=64 bytes)
	.DW 0x03EB					; idVendor (Atmel)
	.DW 0x21FE					; idProduct (Media)
	.DW 0x0000					; bcdDevice
	.DB 1,2,3,1					; iManufacturer, iProduct, iSerialNumber, bNumConfigurations

;	Configuration Descriptor (USB Section 9.6.3)
;	Interface Descriptor (USB Section 9.6.5)
CD:
	.DB 9,0x02					; bLength=9d							bDescriptorType (Configuration)
	.DB ((CD_End - CD)*2 + (HD_End - HD)*2) - 1, 0; 					wTotalLengthL=CD+ID+HD+ED1+ED2-PadByte, wTotalLengthH
	.DB 0x01,0x01				; bNumInterfaces (1)					bConfigValue
	.DB 0x00,0xC0				; iConfiguration						bmAttributes (b7=1 b6=self-powered b5=RWU not supported)
	.DB 100/2,9					; MaxPower (100mA in 2mA units)			[INTERFACE] Length=9d
	.DB 0x04,0					; bDescriptorType (Interface)			bInterfaceNumber (0)
	.DB 0x00,2					; bAlternateSetting						bNumEndpoints (2)
	.DB 0x03,0x00				; bInterfaceClass (HID)					bInterfaceSubClass
	.DB 0x00,0x00				; bInterfaceProtocol					iInterface
CD_End:

;	HID Descriptor (HID Section 6.2.1)
;	Endpoint Descriptors (USB Section 9.6.6)
HD:
	.DB 9,0x21					; bLength=9d							bDescriptorType (HID)
	.DW 0x0111					; bcdHID (HID Rev 1.11)
	.DB 0x00,1					; bCountryCode (none)					bNumDescriptors (1)
	.DB 0x22,((RD_End - RD)*2)-1; bDescriptorType (Report)				wReportDescriptorLengthL

	.DB 0x00,7					; wReportDescriptorLengthH				[ENDPOINT1] bLength=7d
	.DB 0x05,0x81				; bDescriptorType (Endpoint)			bEndpointAddress (IN EP1=0x80|0x01)	
	.DB 0x03,0x20				; bmAttributes (Interrupt)				wMaxPacketSizeL (32 bytes)
	.DB 0x00,0x0A				; wMaxPacketSizeH						bInterval (10 msec)

	.DB 7,0x05					; [ENDPOINT2] bLength=7d				bDescriptorType (Endpoint)	
	.DB 0x02,0x03				; bEndpointAddress (OUT EP2=0x00|0x02)	bmAttributes (Interrupt)
	.DB 0x20,0x00				; wMaxPacketSizeL (32 bytes)			wMaxPacketSizeH
	.DB 0x0A,0x00				; bInterval (10 msec)					*Pad Byte*
HD_End:

;	Report Descriptor (HID Section 6.2.2)
;
;	IN Endpoint Bitmap; the second byte is not used.
;
;			+----+-----+-----+----+-----+----+----+----+
;	Byte1:	|STOP|SCAN-|SCAN+|PLAY|PAUSE|VOL-|VOL+|MUTE|
;			+----+-----+-----+----+-----+----+----+----+
;	Byte2:	| 0  |  0  |  0  | 0  |  0  | 0  | 0  | 0  |
;			+----+-----+-----+----+-----+----+----+----+
;
RD:
	.DB 0x05,0x0C				; Usage Page (Consumer Devices)			05 0C
	.DB 0x09,0x01				; Usage (Consumer Control)				09 01
	.DB 0xA1,0x01				; Collection							A1 01

;	Input Report:
	.DB 0x15,0x00				;   Logical Minimum (0)					15 00
	.DB 0x25,0x01				;   Logical Maximum (1)					25 01
	.DB 0x09,0xE2				;   Usage(Mute)							09 E2
	.DB 0x09,0xE9				; 	Usage(VOL+)							09 E9
	.DB 0x09,0xEA				; 	Usage(VOL-)							09 EA
	.DB 0x09,0xB1				; 	Usage(Pause)						09 B1
	.DB 0x09,0xB0				; 	Usage(Play)							09 B0
	.DB	0x09,0xB5				; 	Usage(Scan Next Track)				09 B5
	.DB	0x09,0xB6				; 	Usage(Scan Previous Track)			09 B6
	.DB 0x09,0xB7				;   Usage(Stop)							09 B7
	.DB 0x75,0x01				; 	Report Size (1)						75 01
	.DB 0x95,0x08				;   Report Count (8)					95 08
	.DB 0x81,0x02				;   Input(Data,Variable,Absolute)		81 02 - Byte1
	.DB 0x81,0x01				;   Input(Constant)						81 01 - Byte2 - all zeros.

;	Output Report:
	.DB 0x09,0x01				;	Usage (Vendor)						09 01
	.DB 0x15,0x00				;   Logical Minimum (-127)				15 00
	.DB	0x25,0x01				;	Logical Maximum (128)				25 01
	.DB	0x75,0x08				;	Report Size (8)						75 08
	.DB	0x95,0x02				; 	Report Count (2)					95 02
	.DB	0x91,0x02				;	Output (Data, Var, Abs)				91 02

	.DB 0xC0,0x00				; End Collection, *Pad Byte*			C0 00
RD_END:


; 	String Descriptors (USB Section 9.6.7).  All strings are 2 byte Unicode
;
;	Language:
SD0:
	.DB (SD0_End - SD0)*2, 0x03		; bLength							bDescriptorType (String)
	.DW 0x0409						; wLangID (US English)	
SD0_End:	

;	Manufacturer ID:
SD1:
	.DB (SD1_End - SD1)*2, 0x03		; bLength							bDescriptorType (string)
	.DW 'G','E','N','E','R','I','C'
SD1_End:

;	Product ID:
SD2:
	.DB (SD2_End - SD2)*2, 0x03		; bLength							bDescriptorType (string)
	.DW 'A','V','R','U','S','B','1','6','2','-','M','E','D','I','A'
SD2_End:

;	Serial Number (Device Instance ID):
SD3:
	.DB (SD3_End - SD3)*2, 0x03		; bLength							bDescriptorType (string)
	.DW '0','0','0','1'
SD3_End:

