;********************************************************************************************************
;
;	CDC Class - Serial Descriptor
;
;	Allows an AT90USB162 to enumerate as a serial device
;
;	Requires an .inf file to cause the correct Windows drivers to be loaded.  A suitable .inf is included
;	with the project files (USBtoSerial.inf).  If the VID/PID are changed in the Device Descriptor, the 
;	.inf VID/PID values must be changed to match.
;
;	Endpoints are defined as follows:
;
;		Control Endpoint	64 bytes	Control (EP0_PACKET_SIZE)
;		Endpoint 1			16 bytes 	Interrupt NOTIFICATION endpoint (device -> host)
;		Endpoint 2			32 bytes	Bulk IN endpoint (device -> host)
;		Endpoint 3			32 bytes	Bulk OUT endpoint (host -> device)
;
;	References to the USB2.0 and CDC1.1 and PSTN1.2 specifications are provided.  The specifications are
;	downloadable from http://www.usb.org/developers/docs.
;
;	NOTE: The AVR compiler arranges memory in words.  If a descriptor length is an uneven number
;	of bytes, a "pad byte" must be included explicitly (if not, the compiler will add one and will generate
;	a warning message).  The descriptor length calculation must take the pad byte into consideration.
;	Length calculations are in words and are multiplied by 2 to get the number of bytes.
;	
;	Bytes are sent to the host COMx via the IN endpoint and are received from the host COMx via the OUT
;	endpoint. Both endpoints must be type "BULK".
;
;	The NOTIFICATION endpoint (type "INTERRUPT") provides serial port status to the host and consists of
;	a 10 byte packet (see CDC11 Section 6.3.5, table 69).  E.g. A1 20 00 00 00 00 02 00 nn nn, where
;	nn nn = UART Bitmap Values from Table 69.
;
;	You must replace the Hid requests with the CDC requests below.  The bRequest values from the CDC1.1
;	specification are as follows (when bmRequestType bit5 = 1):
;
;		Set_Line_Coding (0x20) - a 7 byte packet is sent to the device - see section 6.2.12, Table 50
;		Get_Line_Coding (0x21) - a 7 byte packet is sent to the host - see section 6.2.13, Table 50
;		Set_Control_Line_State (0x22) - see section 6.2.14, table 51
;		Send_Break (0x23) - "break" status - see secton 6.2.15
;
;	Version November 30, 2009
;
;********************************************************************************************************

;	Device_Descriptor (USB Section 9.6.1)
DD:
	.DB 18,0x01						; bLength=18d							bDescriptorType (Device)
	.DW 0x0200						; bcdUSB (USB Rev 2.00)
	.DB 0x00,0x00					; bDeviceClass							bDeviceSubClass 
	.DB 0x00,EP0_PACKET_SIZE		; bDeviceProtocol						bMaxPacketSizeEP0 (64bytes)
	.DW 0x03EB						; idVendor (Atmel)
	.DW 0x21FC						; idProduct (Serial Port)
	.DW 0x0000						; bcdDevice
	.DB 1,2,3,1						; iManufacturer, iProduct, iSerialNumber, bNumConfigurations
	
;	Configuration Descriptor (USB Section 9.6.3)
;	Interface Descriptor (USB Section 9.6.5)
CD:
	.DB	9,0x02						; bLength=9								bDescriptorType (Configuration)
	.DB	((CD_End - CD)*2 - 1), 0	; wTotalLengthL,				 		wTotalLengthH
	.DB	0x02,0x01					; bNumInterfaces (2)					bConfigValue
	.DB	0x00,0xC0					; iConfiguration						bmAttributes (b7=1 b6=self-powered b5=RWU not supported)
	.DB	100/2,0x09					; MaxPower (100mA in 2mA units)			[INTERFACE] bLength=9d

; Communication Class Interface Descriptor (USBCDC11 - Section 4.3, 4.3)
	.DB	0x04,0x00					; bDescriptorType (Interface)			bInterfaceNumber (0)
	.DB	0x00,0x01					; bAlternateSetting						bNumEndpoints
	.DB	0x02,0x02					; bInterfaceClass (CDC)					bInterfaceSubClass (Abstract Control Model)
	.DB	0x00,0x00					; bInterfaceProtocol					iInterface

; CDC Header Functional Descriptor
	.DB	5,0x24						; bFunctionLength=5						bDescriptorType (Interface)
	.DB	0x00,0x10					; bDescriptorSubType					bcdCdcL (CDC Rev 1.10)
	.DB	0x01,0x05					; bcdCdcH								bFunctionLength=5

; CDC Call Management Functional Descriptor
	.DB	0x24,0x01					; bDescriptorType (Interface)			bDescriptorSubType
	.DB	0x01,0x03					; bmCapabilities						bDataInterface (0x03)

; CDC Abstract Control Management Functional Descriptor
	.DB	0x04,0x24					; bFunctionLength=4						bDescriptorType (Interface)
	.DB	0x02,0x06					; bDescriptorSubType (Abstract)			bmCapabilities (b7..b3=0 b2=supports Send Break b1=Set/Get states b0=0)

; CDC Union Functional Descriptor
	.DB	5,0x24						; bFunctionLength						bDescriptorType (Interface)
	.DB	0x06,0x01					; bDescriptorSubType					bControlInterface (0x01)
	.DB	0x00,0x07					; bSubordinateInterface0				[ENDPOINT] bLength=7

; CDC Notification Endpoint
	.DB	0x05,0x81					; bDescriptorType (Endpoint)			bEndpointAddress (IN EP1=0x80|0x01)
	.DB	0x03,0x10					; bmAttributes (Interrupt)				wMaxPacketSizeL	(16 bytes)
	.DB	0x00,0xFF					; wMaxPacketSizeH						bInterval (Infinite)

; Data Interface Class Descriptor (USBCDC11 - Section 4.6)
	.DB	0x09,0x04					; [INTERFACE] bLength=9d				bDescriptorType (Interface)
	.DB	0x01,0x00					; bInterfaceNumber (1)					bAlternateSetting
	.DB	0x02,0x0A					; bNumEndpoints (2)						bInterfaceClass (Data Interface Class)
	.DB	0x00,0x00					; bInterfaceSubClass					bInterfaceProtocol
	.DB	0x00,0x07					; iInterface							[ENDPOINT] bLength=7

; Data IN Endpoint
	.DB	0x05,0x82					; bDescriptorType (Endpoint)			bEndpointAddress (IN EP2=0x80|0x02)
	.DB	0x02,0x20					; bmAttributes (Bulk)					wMaxPacketSizeL (32 bytes)
	.DB	0x00,0x00					; wMaxPacketSizeH						bInterval (0)

; Data OUT Endpoint
	.DB	0x07,0x05					; [ENDPOINT] bLength=7					bDescriptorType (Endpoint)
	.DB	0x03,0x02					; bEndpointAddress (OUT EP3=0x00|0x03)	bmAttributes (Bulk)
	.DB 0x20,0x00					; wMaxPacketSizeL (32 bytes)			wMaxPacketSizeH
	.DB 0x00,0x00					; bInterval (Infinite)					*Pad Byte*
CD_End:


; 	String Descriptors (USB Section 9.6.7).  All strings are 2 byte Unicode.
;
;	Language:
SD0:
	.DB 4,0x03						; bLength		bDescriptorType (String)
	.DW 0x0409						; wLangID (US English)	
SD0_End:	

;	Manufacturer ID:
SD1:
	.DB (SD1_End - SD1)*2, 0x03		; bLength		bDescriptorType (String)
	.DW 'G','E','N','E','R','I','C'
SD1_End:

;	Product ID:
SD2:
	.DB (SD2_End - SD2)*2, 0x03		; bLength		bDescriptorType (String)
	.DW 'A','T','9','0','U','S','B','1','6','2','-','S','E','R','I','A','L'
SD2_End:

;	Serial Number (Device Instance ID):
SD3:
	.DB (SD3_End - SD3)*2, 0x03
	.DW '0','0','0','1'
SD3_End:
