;******************************************************************************
;
;	USART Macros for the AT90USB162
;
;	Baud Rate calculations are dependent on CPU clock speed.  Refer to the
;	datasheet to confirm min/max baud rates for CPU clock speed and choice
;	of _1x vs. _2x option (but in general, use _1x for lower baud rates and
;	_2x for higher baud rates).
;
;	The USART initialization assumes: 8 bits, 1 stop bit, no parity
;	No flow control has been implemented (e.g., no RTS/CTS).
;
;	Macros:
;
;		ConfigureUSART_1x	baudrate		- baudrates 300 - 57600
;		ConfigureUSART_2x	baudrate		- baudrates 9600 - 115200
;		PutChrI				k, TRUE|FALSE	- print literal character, newline=True/False
;		PutChr				r, TRUE|FALSE	- print register character, newline=True/False
;		PutHexI				k, TRUE|FALSE	- print literal hex, newline=True/False
;		PutHex				r, TRUE|FALSE	- print register hex, newline=True/False
;
;	Version: November 22, 2009
;
;******************************************************************************
.EQU	F_OSC					= 16000000	; change this if using a different clock frequency (e.g. 8000000)
.EQU	FALSE					= 0x00
.EQU	TRUE					= 0xFF
.EQU	SPACE					= 0x20
.EQU	LINE_FEED				= 0x0A
.EQU	CARRIAGE_RETURN			= 0x0D
.EQU	NULL					= 0x00

;
; 	Configure USART using 1x calculation - better for low baud rates
;
.macro	ConfigureUSART_1x					; ConfigureUSART_1x baudrate		baudrate={300, 1200, 9600, 19200, 57600}
	ldi		temp, (0<<U2X1)					; single speed transfer rate
	sts		UCSR1A, temp
	ldi		temp, HIGH((F_OSC /(16 * @0)) - 1)
	sts 	UBRR1H, temp
	ldi		temp, LOW((F_OSC /(16 * @0)) - 1)
	sts 	UBRR1L, temp
	ldi		temp, (1<<UCSZ11 | 1<<UCSZ10)	; 8 bits, 1 Stop bit, No parity
	sts		UCSR1C,temp
	ldi		temp,(1<<TXEN1)					; enable USART transmitter
	sts		UCSR1B,temp
.endm

;
; 	Configure USART using 2x calculation - better for high baud rates
;
.macro	ConfigureUSART_2x					; ConfigureUSART_2x baudrate		baudrate={9600, 19200, 57600, 115200}
	ldi		temp, (1<<U2X1)					; double speed transfer rate
	sts		UCSR1A, temp
	ldi		temp, HIGH((F_OSC /(8 * @0)) - 1)
	sts 	UBRR1H, temp
	ldi		temp, LOW((F_OSC /(8 * @0)) - 1)
	sts 	UBRR1L, temp
	ldi		temp, (1<<UCSZ11 | 1<<UCSZ10)	; 8 bits, 1 Stop bit, No parity
	sts		UCSR1C,temp
	ldi		temp,(1<<TXEN1)					; enable USART transmitter
	sts		UCSR1B,temp
.endm

;
;	Put Character Immediate
;
;	Display a character
;
.macro 	PutChrI								; PutChrI k, newline		k={literal}, newline={TRUE | FALSE}
	ldi		usartByte,@0
	rcall	Usart_TxChr
.if (@1==TRUE)								; if the second parameter is "TRUE", then send a new line sequence
	ldi		usartByte,LINE_FEED
	rcall	Usart_TxChr
	ldi		usartByte,CARRIAGE_RETURN
	rcall	Usart_TxChr
.endif
.endmacro

;
;	Put Character
;
;	Display a character contained in register r
;
.macro 	PutChr								; PutChr r, newline			r={r0..r31}, newline={TRUE | FALSE}
	mov		usartByte,@0
	rcall	Usart_TxChr
.if (@1==TRUE)								; if the second parameter is "TRUE", then send a new line sequence
	ldi		usartByte,LINE_FEED
	rcall	Usart_TxChr
	ldi		usartByte,CARRIAGE_RETURN
	rcall	Usart_TxChr
.endif
.endmacro

;
;	Put Hex Immediate
;
;	Display a byte as two hexadecimal characters
;
.macro 	PutHexI								; PutHexI k, newline		k={literal}, newline={TRUE | FALSE}
	ldi		usartByte,@0
	rcall	Usart_TxHex
.if (@1==TRUE)								; if the second parameter is "TRUE", then send a new line sequence
	ldi		usartByte,LINE_FEED
	rcall	Usart_TxChr
	ldi		usartByte,CARRIAGE_RETURN
	rcall	Usart_TxChr
.endif
.endmacro

;
;	Put Hex
;
;	Display an byte contained in register r as two hexadecimal characters
;
.macro 	PutHex								; PutHex r, newline			r={r0..r31}, newline={TRUE | FALSE}
	mov		usartByte,@0
	rcall	Usart_TxHex
.if (@1==TRUE)								; if the second parameter is "TRUE", then send a new line sequence
	ldi		usartByte,LINE_FEED
	rcall	Usart_TxChr
	ldi		usartByte,CARRIAGE_RETURN
	rcall	Usart_TxChr
.endif
.endmacro


