;*****************************************************************************
;
;	USB Macros for the AT90USB82/162 or the ATMega8/16/32U2
;
;	Notes: 	
;
;	1) Most of the USB interface registers are located outside of the
;	range of the in/out instructions and therefore the lds/sts instructions
;	must be used.
;
;	2) The temp register (r16) is used in all macros except the Read/Write FIFO
;	macros.
;
;	3) The sum total of all endpoints memory allocation cannot exceed 176
;	bytes.
;
;	Version: November 24, 2009
;
;*****************************************************************************

; PLL clock prescaler:
.EQU	PLL_x3   				= (0<<PLLP2 | 0<<PLLP1 | 1<<PLLP0) 			; 16 MHz external clock multiplier (x3)
.EQU	PLL_x6					= (0<<PLLP2 | 0<<PLLP1 | 0<<PLLP0)			;  8 MHz external clock multiplier (x6)
.EQU	PLL_PRESCALE			= PLL_x3									; select prescale according to CPU clock frequency

; Endpoint macro parameters
.EQU	EP_0					= (0<<EPNUM2 | 0<<EPNUM1 | 0<<EPNUM0)		; Control endpoint
.EQU	EP_1					= (0<<EPNUM2 | 0<<EPNUM1 | 1<<EPNUM0)		; endpoint number 1
.EQU	EP_2					= (0<<EPNUM2 | 1<<EPNUM1 | 0<<EPNUM0)		; endpoint number 2
.EQU	EP_3					= (0<<EPNUM2 | 1<<EPNUM1 | 1<<EPNUM0)		; endpoint number 3
.EQU	EP_4					= (1<<EPNUM2 | 0<<EPNUM1 | 0<<EPNUM0)		; endpoint number 4
.EQU	EP_SIZE8				= (0<<EPSIZE1 | 0<<EPSIZE0)					; endpoint size 8 bytes
.EQU	EP_SIZE16				= (0<<EPSIZE1 | 1<<EPSIZE1)					; endpoint size 16 bytes
.EQU	EP_SIZE32				= (1<<EPSIZE1 | 0<<EPSIZE0)					; endpoint size 32 bytes
.EQU	EP_SIZE64				= (1<<EPSIZE1 | 1<<EPSIZE0)					; endpoint size 64 bytes
.EQU	EP_IN					= (1<<EPDIR)								; endpoint direction IN
.EQU	EP_OUT					= (0<<EPDIR)								; endpoint direction OUT
.EQU	EP_CONTROL				= (0<<EPTYPE1 | 0<<EPTYPE0)					; endpoint type CONTROL
.EQU	EP_ISOCHRONOUS			= (0<<EPTYPE1 | 1<<EPTYPE0)					; endpoint type ISOCHRONOUS
.EQU	EP_BULK					= (1<<EPTYPE1 | 0<<EPTYPE0)					; endpoint type BULK
.EQU	EP_INTERRUPT			= (1<<EPTYPE1 | 1<<EPTYPE0)					; endpoint type INTERRUPT
.EQU	EP_ONE_BANK				= (0<<EPBK1 | 0<<EPBK0)						; endpoint one bank
.EQU	EP_TWO_BANKS			= (0<<EPBK1 | 1<<EPBK0)						; endpoint two banks

;----------------------------------------------------------------------------
;	USB Interface Management Macros
;----------------------------------------------------------------------------
; Enable 3.3v Regulator (if bus-powered)
.macro	EnableRegulator
	ldi		temp,(0<<REGDIS)
	sts		REGCR,temp
.endm

; Disable the 3.3v Regulator (if self-powered)
.macro	DisableRegulator
	ldi		temp,(1<<REGDIS)
	sts		REGCR,temp
.endm

; Enable USB interface
.macro	EnableUSB
	lds		temp,USBCON
	sbr		temp,(1<<USBE)
	sts		USBCON,temp
.endm

; Disable USB interface
.macro	DisableUSB
	lds		temp,USBCON
	cbr		temp,(1<<USBE)
	sts		USBCON,temp
.endm

; Attach to USB bus without resetting the CPU
.macro	AttachUSB
	ldi		temp,(0<<RSTCPU | 0<<DETACH)
	sts		UDCON,temp
.endm

; Detach from USB bus without resetting the CPU
.macro	DetachUSB
	ldi		temp,(0<<RSTCPU | 1<<DETACH)
	sts		UDCON,temp
.endm

;----------------------------------------------------------------------------
;	USB Clock Macros
;----------------------------------------------------------------------------
;  Enable (Unfreeze) USB clock
.macro	EnableUSBClock
	lds		temp,USBCON
	cbr		temp,(1<<FRZCLK)
	sts		USBCON,temp
.endm

; Disable (Freeze) USB clock
.macro	DisableUSBClock
	lds		temp,USBCON
	sbr		temp,(1<<FRZCLK)
	sts		USBCON,temp
.endm

;----------------------------------------------------------------------------
;	USB PLL Macros
;----------------------------------------------------------------------------
; Configure the PLL prescaler
.macro	SetPLLPrescale
	ldi		temp,PLL_PRESCALE
	out		PLLCSR,temp
.endm

; Enable the PLL
.macro	EnablePLL
	in		temp,PLLCSR
	sbr		temp,(1<<PLLE)
	out		PLLCSR,temp								
_wt:in		temp,PLLCSR
	sbrs	temp,PLOCK					; wait until PLL lock bit set
	rjmp	_wt
.endm

; Disable the PLL
.macro	DisablePLL
	in		temp,PLLCSR
	cbr		temp,(1<<PLLE)
	out		PLLCSR,temp
.endm

;----------------------------------------------------------------------------
;	USB Device Macros
;----------------------------------------------------------------------------
.macro	EnableSUSPEND
	lds		temp,UDIEN
	sbr		temp,(1<<SUSPE)
	sts		UDIEN,temp
.endm

.macro	DisableSUSPEND
	lds		temp,UDIEN
	cbr		temp,(1<<SUSPE)
	sts		UDIEN,temp
.endm

.macro	ClearSUSPEND
	lds		temp,UDINT
	cbr		temp,(1<<SUSPI)
	sts		UDINT,temp
.endm

.macro	EnableWAKEUP
	lds		temp,UDIEN
	sbr		temp,(1<<WAKEUPE)
	sts		UDIEN,temp
.endm

.macro	DisableWAKEUP
	lds		temp,UDIEN
	cbr		temp,(1<<WAKEUPE)
	sts		UDIEN,temp
.endm

.macro	ClearWAKEUP
	lds		temp,UDINT
	cbr		temp,(1<<WAKEUPI)
	sts		UDINT,temp
.endm

.macro	EnableRESET
	lds		temp,UDIEN
	sbr		temp,(1<<EORSTE)
	sts		UDIEN,temp
.endm
.macro	DisableRESET
	lds		temp,UDIEN
	cbr		temp,(1<<EORSTE)
	sts		UDIEN,temp
.endm

.macro	ClearRESET
	lds		temp,UDINT
	cbr		temp,(1<<EORSTI)
	sts		UDINT,temp
.endm

.macro	EnableSOF
	lds		temp,UDIEN
	sbr		temp,(1<<SOFE)
	sts		UDIEN,temp
.endm

.macro	DisableSOF
	lds		temp,UDIEN
	cbr		temp,(1<<SOFE)
	sts		UDIEN,temp
.endm

.macro	ClearSOF
	lds		temp,UDINT
	cbr		temp,(1<<SOFI)
	sts		UDINT,temp
.endm

;----------------------------------------------------------------------------
;	USB Endpoint Macros
;----------------------------------------------------------------------------
; Select the endpoint number to work with
.macro	SelectEndpoint					; SelectEndpoint EP_n where n = {0,1,2,3,4}
	ldi		temp,@0
	sts		UENUM,temp
.endm

; Reset the selected endpoint
.macro	ResetEndpoint					; ResetEndpoint EP_n where n = {0,1,2,3,4}
	ldi		temp,(1<<@0)
	sts		UERST,temp
	ldi		temp,(0<<@0)
	sts		UERST,temp
.endm

; Set the current endpoint type and direction
.macro	SetEndpointType					; SelectEndpointType (EP_t | EP_d) where t = {CONTROL, INTERRUPT, BULK, ISOCHRONOUS}, d = {IN, OUT}
	lds		temp,UECFG0X
	sbr		temp,@0
	sts		UECFG0X,temp
.endm

; Set the current endpoint size and number of banks
.macro	SetEndpointSize					; SelectEndpointSize (EP_s | EP_b) where s = {SIZE8, SIZE16, SIZE32, SIZE64}, b = {ONE_BANK, TWO_BANKS}
	lds		temp,UECFG1X
	sbr		temp,@0
	sts		UECFG1X,temp
.endm

; Enable the current endpoint
.macro	EnableEndpoint
	ldi		temp,(1<<EPEN)
	sts		UECONX,temp
.endm

; Allocate the current configuration in endpoint RAM
.macro	AllocateEndpointMemory
	lds		temp,UECFG1X
	sbr		temp,(1<<ALLOC)
	sts		UECFG1X,temp
.endm

; Test if the current endpoint is configured.  0x00=False, 0x80=True
.macro	IsEndpointConfigured
	lds		temp,UESTA0X
	andi	temp,(1<<CFGOK)
.endm

;----------------------------------------------------------------------------
;	USB Address Macros
;----------------------------------------------------------------------------
; Set the USB device address
.macro	SetAddress
	mov		temp,@0
	andi	temp,0x7F					; mask address to 7 bits.
	sts		UDADDR,temp
.endm

; Enable USB device address
.macro	EnableAddress
	lds		temp,UDADDR
	sbr		temp,(1<<ADDEN)
	sts		UDADDR,temp
.endm

;----------------------------------------------------------------------------
;	USB Interrupt Macros
;----------------------------------------------------------------------------
.macro	EnableSETUP
	lds		temp,UEIENX
	sbr		temp,(1<<RXSTPE)
	sts		UEIENX,temp
.endm

.macro	ClearSETUP
	lds		temp,UEINTX
	cbr		temp,(1<<RXSTPI)
	sts		UEINTX,temp
.endm

.macro	EnableOUT
	lds		temp,UEIENX
	sbr		temp,(1<<RXOUTE)
	sts		UEIENX,temp
.endm

.macro	ClearOUT
	lds		temp,UEINTX
	cbr		temp,(1<<RXOUTI)
	sts		UEINTX,temp
.endm

.macro	EnableIN
	lds		temp,UEIENX
	sbr		temp,(1<<TXINE)
	sts		UEIENX,temp
.endm

.macro	ClearIN
	lds		temp,UEINTX
	cbr		temp,(1<<TXINI)
	sts		UEINTX,temp	
.endm

.macro	EnableNAKIN
	lds		temp,UEIENX
	sbr		temp,(1<<NAKINE)
	sts		UEIENX,temp
.endm

.macro	ClearNAKIN
	lds		temp,UEINTX
	cbr		temp,(1<<NAKINI)
	sts		UEINTX,temp
.endm

.macro	EnableNAKOUT
	lds		temp,UEIENX
	sbr		temp,(1<<NAKOUTE)
	sts		UEIENX,temp
.endm

.macro	ClearNAKOUT
	lds		temp,UEINTX
	cbr		temp,(1<<NAKOUTI)
	sts		UEINTX,temp
.endm

; Enable a Stall handshake until a Setup occurs for the endpoint
.macro	SetSTALL
	lds		temp,UECONX
	sbr		temp,(1<<STALLRQ)
	sts		UECONX,temp
.endm

.macro	CancelSTALL
	lds		temp,UECONX
	sbr		temp,(1<<STALLRQC)
	sts		UECONX,temp
.endm

.macro	ClearSTALL
	lds		temp,UEINTX
	cbr		temp,(1<<STALLEDI)
	sts		UEINTX,temp
.endm

.macro	ClearFIFO
	lds		temp,UEINTX
	cbr		temp,(1<<FIFOCON)
	sts		UEINTX,temp
.endm

.macro	WaitINReady
_wt:lds		temp,UEINTX
	sbrs	temp,TXINI
	rjmp	_wt
.endm

.macro	WaitOUTReceived
_wt:lds		temp,UEINTX
	sbrs	temp,RXOUTI
	rjmp	_wt
.endm

.macro	WaitReadWriteAllowed
_wt:lds		temp,UEINTX
	sbrs	temp,RWAL
	rjmp	_wt
.endm

.macro	WaitNAKIN
_wt:lds		temp,UEINTX
	sbrs	temp,NAKINI
	rjmp	_wt
.endm

.macro	WaitNAKOUT
_wt:lds		temp,UEINTX
	sbrs	temp,NAKOUTI
	rjmp	_wt
.endm

;----------------------------------------------------------------------------
;	USB Endpoint FIFO Macros
;----------------------------------------------------------------------------
;	Read a byte from the current endpoint FIFO
.macro	ReadByte						; ReadByte r where r = {r0..r31}
	lds		@0,UEDATX
.endm

;	Write a byte to the current endpoint FIFO
.macro	WriteByte						; WriteByte r where r = {r0..r31}
	sts		UEDATX,@0
.endm

;	Return the number of bytes in the current endpoint FIFO
.macro	GetByteCount
	lds		@0,UEBCLX					; EPByteCount r where r = {r0..r31}
.endm

